@@ -1,7 +1,7 @@
-{-# LANGUAGE BangPatterns #-}
+{-# LANGUAGE BangPatterns, ScopedTypeVariables #-}
 -- |
 -- Module    : Numeric.SpecFunctions
--- Copyright : (c) 2009, 2011 Bryan O'Sullivan
+-- Copyright : (c) 2009, 2011, 2012 Bryan O'Sullivan
 -- License   : BSD3
 --
 -- Maintainer  : bos@serpentine.com
@@ -10,11 +10,17 @@
 --
 -- Special functions and factorials.
 module Numeric.SpecFunctions (
+    -- * Error function
+    erf
+  , erfc
+  , invErf
+  , invErfc
     -- * Gamma function
-    logGamma
+  , logGamma
   , logGammaL
   , incompleteGamma
   , invIncompleteGamma
+  , digamma
     -- * Beta function
   , logBeta
   , incompleteBeta
@@ -35,13 +41,67 @@ module Numeric.SpecFunctions (
 
 import Data.Bits       ((.&.), (.|.), shiftR)
 import Data.Int        (Int64)
-import Data.Word       (Word64)
-import Data.Number.Erf (erfc)
+import qualified Data.Number.Erf     as Erf (erfc,erf)
 import qualified Data.Vector.Unboxed as U
 
 import Numeric.Polynomial.Chebyshev    (chebyshevBroucke)
-import Numeric.MathFunctions.Constants (m_epsilon, m_sqrt_2_pi, m_ln_sqrt_2_pi, 
-                                        m_NaN, m_neg_inf, m_pos_inf, m_sqrt_2)
+import Numeric.Polynomial              (evaluateEvenPolynomialL,evaluateOddPolynomialL)
+import Numeric.MathFunctions.Constants ( m_epsilon, m_NaN, m_neg_inf, m_pos_inf
+                                       , m_sqrt_2_pi, m_ln_sqrt_2_pi, m_sqrt_2
+                                       , m_eulerMascheroni
+                                       )
+import Text.Printf
+
+
+----------------------------------------------------------------
+-- Error function
+----------------------------------------------------------------
+
+-- | Error function.
+--
+-- > erf -∞ = -1
+-- > erf  0 =  0
+-- > erf +∞ =  1
+erf :: Double -> Double
+{-# INLINE erf #-}
+erf = Erf.erf
+
+-- | Complementary error function.
+--
+-- > erfc -∞ = 2
+-- > erfc  0 = 1
+-- > errc +∞ = 0
+erfc :: Double -> Double
+{-# INLINE erfc #-}
+erfc = Erf.erfc
+
+
+-- | Inverse of 'erf'.
+invErf :: Double -- ^ /p/ ∈ [-1,1]
+       -> Double
+invErf p = invErfc (1 - p)
+
+-- | Inverse of 'erfc'.
+invErfc :: Double -- ^ /p/ ∈ [0,2]
+        -> Double
+invErfc p
+  | p == 2        = m_neg_inf
+  | p == 0        = m_pos_inf
+  | p >0 && p < 2 = if p <= 1 then r else -r
+  | otherwise     = modErr $ "invErfc: p must be in [0,2] got " ++ show p
+  where
+    pp = if p <= 1 then p else 2 - p
+    t  = sqrt $ -2 * log( 0.5 * pp)
+    -- Initial guess
+    x0 = -0.70711 * ((2.30753 + t * 0.27061) / (1 + t * (0.99229 + t * 0.04481)) - t)
+    r  = loop 0 x0
+    --
+    loop :: Int -> Double -> Double
+    loop !j !x
+      | j >= 2    = x
+      | otherwise = let err = erfc x - pp
+                        x'  = x + err / (1.12837916709551257 * exp(-x * x) - x * err) -- // Halley
+                    in loop (j+1) x'
 
 
 
@@ -51,18 +111,21 @@ import Numeric.MathFunctions.Constants (m_epsilon, m_sqrt_2_pi, m_ln_sqrt_2_pi,
 
 -- Adapted from http://people.sc.fsu.edu/~burkardt/f_src/asa245/asa245.html
 
--- | Compute the logarithm of the gamma function &#915;(/x/).  Uses
+-- | Compute the logarithm of the gamma function Γ(/x/).  Uses
 -- Algorithm AS 245 by Macleod.
 --
--- Gives an accuracy of 10&#8211;12 significant decimal digits, except
+-- Gives an accuracy of 10-12 significant decimal digits, except
 -- for small regions around /x/ = 1 and /x/ = 2, where the function
 -- goes to zero.  For greater accuracy, use 'logGammaL'.
 --
--- Returns &#8734; if the input is outside of the range (0 < /x/
--- &#8804; 1e305).
+-- Returns ∞ if the input is outside of the range (0 < /x/ ≤ 1e305).
 logGamma :: Double -> Double
 logGamma x
     | x <= 0    = m_pos_inf
+    -- Handle positive infinity. logGamma overflows before 1e308 so
+    -- it's safe
+    | x > 1e308 = m_pos_inf
+    -- Normal cases
     | x < 1.5   = a + c *
                   ((((r1_4 * b + r1_3) * b + r1_2) * b + r1_1) * b + r1_0) /
                   ((((b + r1_8) * b + r1_7) * b + r1_6) * b + r1_5)
@@ -71,7 +134,7 @@ logGamma x
                   ((((x + r2_8) * x + r2_7) * x + r2_6) * x + r2_5)
     | x < 12    = ((((r3_4 * x + r3_3) * x + r3_2) * x + r3_1) * x + r3_0) /
                   ((((x + r3_8) * x + r3_7) * x + r3_6) * x + r3_5)
-    | x > 5.1e5 = k
+    | x > 3e6   = k
     | otherwise = k + x1 *
                   ((r4_2 * x2 + r4_1) * x2 + r4_0) /
                   ((x2 + r4_4) * x2 + r4_3)
@@ -119,6 +182,8 @@ data L = L {-# UNPACK #-} !Double {-# UNPACK #-} !Double
 logGammaL :: Double -> Double
 logGammaL x
     | x <= 0    = m_pos_inf
+    -- Lanroz approximation loses precision for small arguments
+    | x <= 1e-3 = logGamma x
     | otherwise = fini . U.foldl' go (L 0 (x+7)) $ a
     where fini (L l _) = log (l+a0) + log m_sqrt_2_pi - x65 + (x-0.5) * log x65
           go (L l t) k = L (l + k / t) (t-1)
@@ -162,23 +227,46 @@ logGammaCorrection x
 
 
 -- | Compute the normalized lower incomplete gamma function
--- &#947;(/s/,/x/). Normalization means that
--- &#947;(/s/,&#8734;)=1. Uses Algorithm AS 239 by Shea.
-incompleteGamma :: Double       -- ^ /s/
-                -> Double       -- ^ /x/
+-- γ(/s/,/x/). Normalization means that
+-- γ(/s/,∞)=1. Uses Algorithm AS 239 by Shea.
+incompleteGamma :: Double       -- ^ /s/ ∈ (0,∞)
+                -> Double       -- ^ /x/ ∈ (0,∞)
                 -> Double
 incompleteGamma p x
-    | x < 0 || p <= 0 = m_pos_inf
-    | x == 0          = 0
-    | p >= 1000       = norm (3 * sqrt p * ((x/p) ** (1/3) + 1/(9*p) - 1))
-    | x >= 1e8        = 1
-    | x <= 1 || x < p = let a = p * log x - x - logGamma (p + 1)
-                            g = a + log (pearson p 1 1)
-                        in if g > limit then exp g else 0
-    | otherwise       = let g = p * log x - x - logGamma p + log cf
-                        in if g > limit then 1 - exp g else 1
+    | isNaN p || isNaN x = m_NaN
+    | x < 0 || p <= 0    = m_pos_inf
+    | x == 0             = 0
+    -- For very large `p' normal approximation gives <1e-10 error
+    | p >= 2e5           = norm (3 * sqrt p * ((x/p) ** (1/3) + 1/(9*p) - 1))
+    | p >= 500           = approx
+    -- Dubious approximation
+    | x >= 1e8           = 1
+    | x <= 1 || x < p    = let a = p * log x - x - logGamma (p + 1)
+                               g = a + log (pearson p 1 1)
+                           in if g > limit then exp g else 0
+    | otherwise          = let g = p * log x - x - logGamma p + log cf
+                           in if g > limit then 1 - exp g else 1
   where
-    norm a = erfc (- a / m_sqrt_2)
+    -- CDF for standard normal distributions
+    norm a = 0.5 * erfc (- a / m_sqrt_2)
+    -- For large values of `p' we use 18-point Gauss-Legendre
+    -- integration.
+    approx
+      | ans > 0   = 1 - ans
+      | otherwise = -ans
+      where
+        -- Set upper limit for integration
+        xu | x > p1    =         (p1 + 11.5*sqrtP1) `max` (x + 6*sqrtP1)
+           | otherwise = max 0 $ (p1 -  7.5*sqrtP1) `min` (x - 5*sqrtP1)
+        s = U.sum $ U.zipWith go coefY coefW
+        go y w = let t = x + (xu - x)*y
+                 in w * exp( -(t-p1) + p1*(log t - lnP1) )
+        ans = s * (xu - x) * exp( p1 * (lnP1 - 1) - logGamma p)
+        --
+        p1     = p - 1
+        lnP1   = log  p1
+        sqrtP1 = sqrt p1
+    --
     pearson !a !c !g
         | c' <= tolerance = g'
         | otherwise       = pearson a' c' g'
@@ -215,15 +303,14 @@ incompleteGamma p x
 --   approximately holds:
 --
 -- > invIncompleteGamma s . incompleteGamma s = id
---
---   For @invIncompleteGamma s p@ /s/ must be positive and /p/ must be
---   in [0,1] range.
-invIncompleteGamma :: Double -> Double -> Double
+invIncompleteGamma :: Double    -- ^ /s/ ∈ (0,∞)
+                   -> Double    -- ^ /p/ ∈ [0,1]
+                   -> Double
 invIncompleteGamma a p
-  | a <= 0         = 
-      error $ "Statistics.Math.invIncompleteGamma: a must be positive. Got: " ++ show a
-  | p < 0 || p > 1 = 
-      error $ "Statistics.Math.invIncompleteGamma: p must be in [0,1] range. Got: " ++ show p
+  | a <= 0         =
+      modErr $ printf "invIncompleteGamma: a must be positive. a=%g p=%g" a p
+  | p < 0 || p > 1 =
+      modErr $ printf "invIncompleteGamma: p must be in [0,1] range. a=%g p=%g" a p
   | p == 0         = 0
   | p == 1         = 1 / 0
   | otherwise      = loop 0 guess
@@ -231,27 +318,28 @@ invIncompleteGamma a p
     -- Solve equation γ(a,x) = p using Halley method
     loop :: Int -> Double -> Double
     loop i x
-      | i >= 12   = x
-      | otherwise =
-         let 
-           -- Value of γ(a,x) - p
-           f    = incompleteGamma a x - p
-           -- dγ(a,x)/dx
-           f'   | a > 1     = afac * exp( -(x - a1) + a1 * (log x - lna1))
-                | otherwise = exp( -x + a1 * log x - gln)
-           u    = f / f'
-           -- Halley correction to Newton-Rapson step
-           corr = u * (a1 / x - 1)
-           dx   = u / (1 - 0.5 * min 1.0 corr)
-           -- New approximation to x
-           x'   | x < dx    = 0.5 * x -- Do not go below 0
-                | otherwise = x - dx
-         in if abs dx < eps * x'
-            then x'
-            else loop (i+1) x'
+      | i >= 12           = x'
+      -- For small s derivative becomes approximately 1/x*exp(-x) and
+      -- skyrockets for small x. If it happens correct answer is 0.
+      | isInfinite f'     = 0
+      | abs dx < eps * x' = x'
+      | otherwise         = loop (i + 1) x'
+      where
+        -- Value of γ(a,x) - p
+        f    = incompleteGamma a x - p
+        -- dγ(a,x)/dx
+        f'   | a > 1     = afac * exp( -(x - a1) + a1 * (log x - lna1))
+             | otherwise = exp( -x + a1 * log x - gln)
+        u    = f / f'
+        -- Halley correction to Newton-Rapson step
+        corr = u * (a1 / x - 1)
+        dx   = u / (1 - 0.5 * min 1.0 corr)
+        -- New approximation to x
+        x'   | x < dx    = 0.5 * x -- Do not go below 0
+             | otherwise = x - dx
     -- Calculate inital guess for root
     guess
-      -- 
+      --
       | a > 1   =
          let t  = sqrt $ -2 * log(if p < 0.5 then p else 1 - p)
              x1 = (2.30753 + t * 0.27061) / (1 + t * (0.99229 + t * 0.04481)) - t
@@ -297,7 +385,8 @@ logBeta a b
       c   = logGammaCorrection q - logGammaCorrection pq
 
 -- | Regularized incomplete beta function. Uses algorithm AS63 by
---   Majumder abd Bhattachrjee.
+-- Majumder and Bhattachrjee and quadrature approximation for large
+-- /p/ and /q/.
 incompleteBeta :: Double -- ^ /p/ > 0
                -> Double -- ^ /q/ > 0
                -> Double -- ^ /x/, must lie in [0,1] range
@@ -306,28 +395,60 @@ incompleteBeta p q = incompleteBeta_ (logBeta p q) p q
 
 -- | Regularized incomplete beta function. Same as 'incompleteBeta'
 -- but also takes logarithm of beta function as parameter.
-incompleteBeta_ :: Double -- ^ logarithm of beta function
+incompleteBeta_ :: Double -- ^ logarithm of beta function for given /p/ and /q/
                 -> Double -- ^ /p/ > 0
                 -> Double -- ^ /q/ > 0
                 -> Double -- ^ /x/, must lie in [0,1] range
                 -> Double
 incompleteBeta_ beta p q x
-  | p <= 0 || q <= 0 = error "p <= 0 || q <= 0"
-  | x <  0 || x >  1 = error "x <  0 || x >  1"
-  | x == 0 || x == 1 = x
+  | p <= 0 || q <= 0            =
+      modErr $ printf "incompleteBeta_: p <= 0 || q <= 0. p=%g q=%g x=%g" p q x
+  | x <  0 || x >  1 || isNaN x =
+      modErr $ printf "incompletBeta_: x out of [0,1] range. p=%g q=%g x=%g" p q x
+  | x == 0 || x == 1            = x
   | p >= (p+q) * x   = incompleteBetaWorker beta p q x
   | otherwise        = 1 - incompleteBetaWorker beta q p (1 - x)
 
+
+-- Approximation of incomplete beta by quandrature.
+--
+-- Note that x =< p/(p+q)
+incompleteBetaApprox :: Double -> Double -> Double -> Double -> Double
+incompleteBetaApprox beta p q x
+  | ans > 0   = 1 - ans
+  | otherwise = -ans
+  where
+    -- Constants
+    p1    = p - 1
+    q1    = q - 1
+    mu    = p / (p + q)
+    lnmu  = log mu
+    lnmuc = log (1 - mu)
+    -- Upper limit for integration
+    xu = max 0 $ min (mu - 10*t) (x - 5*t)
+       where
+         t = sqrt $ p*q / ( (p+q) * (p+q) * (p + q + 1) )
+    -- Calculate incomplete beta by quadrature
+    go y w = let t = x + (xu - x) * y
+             in  w * exp( p1 * (log t - lnmu) + q1 * (log(1-t) - lnmuc) )
+    s   = U.sum $ U.zipWith go coefY coefW
+    ans = s * (xu - x) * exp( p1 * lnmu + q1 * lnmuc - beta )
+
+
 -- Worker for incomplete beta function. It is separate function to
 -- avoid confusion with parameter during parameter swapping
 incompleteBetaWorker :: Double -> Double -> Double -> Double -> Double
-incompleteBetaWorker beta p q x = loop (p+q) (truncate $ q + cx * (p+q) :: Int) 1 1 1
+incompleteBetaWorker beta p q x
+  -- For very large p and q this method becomes very slow so another
+  -- method is used.
+  | p > 3000 && q > 3000 = incompleteBetaApprox beta p q x
+  | otherwise            = loop (p+q) (truncate $ q + cx * (p+q)) 1 1 1
   where
     -- Constants
     eps = 1e-15
     cx  = 1 - x
     -- Loop
-    loop psq ns ai term betain
+    loop !psq (ns :: Int) ai term betain
       | done      = betain' * exp( p * log x + (q - 1) * log cx - beta) / p
       | otherwise = loop psq' (ns - 1) (ai + 1) term' betain'
       where
@@ -344,57 +465,106 @@ incompleteBetaWorker beta p q x = loop (p+q) (truncate $ q + cx * (p+q) :: Int)
 
 
 -- | Compute inverse of regularized incomplete beta function. Uses
--- initial approximation from AS109 and Halley method to solve equation.
-invIncompleteBeta :: Double     -- ^ /p/
-                  -> Double     -- ^ /q/
-                  -> Double     -- ^ /a/
+-- initial approximation from AS109, AS64 and Halley method to solve
+-- equation.
+invIncompleteBeta :: Double     -- ^ /p/ > 0
+                  -> Double     -- ^ /q/ > 0
+                  -> Double     -- ^ /a/ ∈ [0,1]
                   -> Double
 invIncompleteBeta p q a
-  | p <= 0 || q <= 0 = error "p <= 0 || q <= 0"
-  | a <  0 || a >  1 = error "bad a"
+  | p <= 0 || q <= 0 =
+      modErr $ printf "invIncompleteBeta p <= 0 || q <= 0.  p=%g q=%g a=%g" p q a
+  | a <  0 || a >  1 =
+      modErr $ printf "invIncompleteBeta x must be in [0,1].  p=%g q=%g a=%g" p q a
   | a == 0 || a == 1 = a
   | a > 0.5          = 1 - invIncompleteBetaWorker (logBeta p q) q p (1 - a)
-  | otherwise        = invIncompleteBetaWorker (logBeta p q) p q a
+  | otherwise        =     invIncompleteBetaWorker (logBeta p q) p q  a
+
 
 invIncompleteBetaWorker :: Double -> Double -> Double -> Double -> Double
-invIncompleteBetaWorker beta p q a = loop (0::Int) guess
+-- NOTE: p <= 0.5.
+invIncompleteBetaWorker beta a b p = loop (0::Int) guess
   where
-    p1 = p - 1
-    q1 = q - 1
+    a1 = a - 1
+    b1 = b - 1
     -- Solve equation using Halley method
     loop !i !x
+      -- We cannot continue at this point so we simply return `x'
       | x == 0 || x == 1             = x
+      -- When derivative becomes infinite we cannot continue
+      -- iterations. It can only happen in vicinity of 0 or 1. It's
+      -- hardly possible to get good answer in such circumstances but
+      -- `x' is already reasonable.
+      | isInfinite f'                = x
+      -- Iterations limit reached. Most of the time solution will
+      -- converge to answer because of discreteness of Double. But
+      -- solution have good precision already.
       | i >= 10                      = x
-      | abs dx <= 16 * m_epsilon * x = x
+      -- Solution converges
+      | abs dx <= 16 * m_epsilon * x = x'
       | otherwise                    = loop (i+1) x'
       where
-        f   = incompleteBeta_ beta p q x - a
-        f'  = exp $ p1 * log x + q1 * log (1 - x) - beta
+        -- Calculate Halley step.
+        f   = incompleteBeta_ beta a b x - p
+        f'  = exp $ a1 * log x + b1 * log (1 - x) - beta
         u   = f / f'
-        dx  = u / (1 - 0.5 * min 1 (u * (p1 / x - q1 / (1 - x))))
+        dx  = u / (1 - 0.5 * min 1 (u * (a1 / x - b1 / (1 - x))))
+        -- Next approximation. If Halley step leads us out of [0,1]
+        -- range we revert to bisection.
         x'  | z < 0     = x / 2
             | z > 1     = (x + 1) / 2
             | otherwise = z
             where z = x - dx
-    -- Calculate initial guess
-    guess 
-      | p > 1 && q > 1 = 
-          let rr = (y*y - 3) / 6
-              ss = 1 / (2*p - 1)
-              tt = 1 / (2*q - 1)
-              hh = 2 / (ss + tt)
-              ww = y * sqrt(hh + rr) / hh - (tt - ss) * (rr + 5/6 - 2 / (3 * hh))
-          in p / (p + q * exp(2 * ww))
-      | t'  <= 0  = 1 - exp( (log((1 - a) * q) + beta) / q )
-      | t'' <= 1  = exp( (log(a * p) + beta) / p )
-      | otherwise = 1 - 2 / (t'' + 1)
+    -- Calculate initial guess. Approximations from AS64, AS109 and
+    -- Numerical recipes are used.
+    --
+    -- Equations are referred to by name of paper and number e.g. [AS64 2]
+    -- In AS64 papers equations are not numbered so they are refered
+    -- to by number of appearance starting from definition of
+    -- incomplete beta.
+    guess
+      -- In this region we use approximation from AS109 (Carter
+      -- approximation). It's reasonably good (2 iterations on
+      -- average)
+      | a > 1 && b > 1 =
+          let r = (y*y - 3) / 6
+              s = 1 / (2*a - 1)
+              t = 1 / (2*b - 1)
+              h = 2 / (s + t)
+              w = y * sqrt(h + r) / h - (t - s) * (r + 5/6 - 2 / (3 * h))
+          in a / (a + b * exp(2 * w))
+      -- Otherwise we revert to approximation from AS64 derived from
+      -- [AS64 2] when it's applicable.
+      --
+      -- It slightly reduces average number of iterations when `a' and
+      -- `b' have different magnitudes.
+      | chi2 > 0 && ratio > 1 = 1 - 2 / (ratio + 1)
+      -- If all else fails we use approximation from "Numerical
+      -- Recipes". It's very similar to approximations [AS64 4,5] but
+      -- it never goes out of [0,1] interval.
+      | otherwise = case () of
+          _| p < t / w  -> (a * p * w) ** (1/a)
+           | otherwise  -> 1 - (b * (1 - p) * w) ** (1/b)
+           where
+             lna = log $ a / (a+b)
+             lnb = log $ b / (a+b)
+             t   = exp( a * lna ) / a
+             u   = exp( b * lnb ) / b
+             w   = t + u
       where
-        r   = sqrt ( - log ( a * a ) )
+        -- Formula [2]
+        ratio = (4*a + 2*b - 2) / chi2
+        -- Quantile of chi-squared distribution. Formula [3].
+        chi2 = 2 * b * (1 - t + y * sqrt t) ** 3
+          where
+            t   = 1 / (9 * b)
+        -- `y' is Hasting's approximation of p'th quantile of standard
+        -- normal distribution.
         y   = r - ( 2.30753 + 0.27061 * r )
-                   / ( 1.0 + ( 0.99229 + 0.04481 * r ) * r )
-        t   = 1 / (9 * q)
-        t'  = 2 * q * (1 - t + y * sqrt t) ** 3
-        t'' = (4*p + 2*q - 2) / t'
+                  / ( 1.0 + ( 0.99229 + 0.04481 * r ) * r )
+          where
+            r = sqrt $ - 2 * log p
+
 
 
 
@@ -446,7 +616,7 @@ log1p x
 -- | /O(log n)/ Compute the logarithm in base 2 of the given value.
 log2 :: Int -> Int
 log2 v0
-    | v0 <= 0   = error "Statistics.Math.log2: invalid input"
+    | v0 <= 0   = modErr $ "log2: negative input, got " ++ show v0
     | otherwise = go 5 0 v0
   where
     go !i !r !v | i == -1        = r
@@ -462,26 +632,28 @@ log2 v0
 -- Factorial
 ----------------------------------------------------------------
 
--- | Compute the factorial function /n/!.  Returns &#8734; if the
+-- | Compute the factorial function /n/!.  Returns +∞ if the
 -- input is above 170 (above which the result cannot be represented by
 -- a 64-bit 'Double').
 factorial :: Int -> Double
 factorial n
-    | n < 0     = error "Statistics.Math.factorial: negative input"
+    | n < 0     = error "Numeric.SpecFunctions.factorial: negative input"
     | n <= 1    = 1
     | n <= 170  = U.product $ U.map fromIntegral $ U.enumFromTo 2 n
     | otherwise = m_pos_inf
 
 -- | Compute the natural logarithm of the factorial function.  Gives
 -- 16 decimal digits of precision.
-logFactorial :: Int -> Double
+logFactorial :: Integral a => a -> Double
 logFactorial n
-    | n <= 14   = log (factorial n)
+    | n <  0    = error "Numeric.SpecFunctions.logFactorial: negative input"
+    | n <= 14   = log $ factorial $ fromIntegral n
     | otherwise = (x - 0.5) * log x - x + 9.1893853320467e-1 + z / x
-    where x = fromIntegral (n + 1)
+    where x = fromIntegral n + 1
           y = 1 / (x * x)
           z = ((-(5.95238095238e-4 * y) + 7.936500793651e-4) * y -
                2.7777777777778e-3) * y + 8.3333333333333e-2
+{-# SPECIALIZE logFactorial :: Int -> Double #-}
 
 -- | Calculate the error term of the Stirling approximation.  This is
 -- only defined for non-negative values.
@@ -493,12 +665,11 @@ stirlingError n
                     (i,0) -> sfe `U.unsafeIndex` i
                     _     -> logGamma (n+1.0) - (n+0.5) * log n + n -
                              m_ln_sqrt_2_pi
-  | n > 500     = (s0-s1/nn)/n
-  | n > 80      = (s0-(s1-s2/nn)/nn)/n
-  | n > 35      = (s0-(s1-(s2-s3/nn)/nn)/nn)/n
-  | otherwise   = (s0-(s1-(s2-(s3-s4/nn)/nn)/nn)/nn)/n
+  | n > 500     = evaluateOddPolynomialL (1/n) [s0,-s1]
+  | n > 80      = evaluateOddPolynomialL (1/n) [s0,-s1,s2]
+  | n > 35      = evaluateOddPolynomialL (1/n) [s0,-s1,s2,-s3]
+  | otherwise   = evaluateOddPolynomialL (1/n) [s0,-s1,s2,-s3,s4]
   where
-    nn = n*n
     s0 = 0.083333333333333333333        -- 1/12
     s1 = 0.00277777777777777777778      -- 1/360
     s2 = 0.00079365079365079365079365   -- 1/1260
@@ -557,11 +728,92 @@ n `choose` k
     max64          = fromIntegral (maxBound :: Int64)
     round64 x      = round x :: Int64
 
+-- | Compute ψ0(/x/), the first logarithmic derivative of the gamma
+-- function. Uses Algorithm AS 103 by Bernardo, based on Minka's C
+-- implementation.
+digamma :: Double -> Double
+digamma x
+    | isNaN x || isInfinite x                  = m_NaN
+    -- FIXME:
+    --   This is ugly. We are testing here that number is in fact
+    --   integer. It's somewhat tricky question to answer. When ε for
+    --   given number becomes 1 or greater every number is represents
+    --   an integer. We also must make sure that excess precision
+    --   won't bite us.
+    | x <= 0 && fromIntegral (truncate x :: Int64) == x = m_neg_inf
+    -- Jeffery's reflection formula
+    | x < 0     = digamma (1 - x) + pi / tan (negate pi * x)
+    | x <= 1e-6 = - γ - 1/x + trigamma1 * x
+    | x' < c    = r
+    -- De Moivre's expansion
+    | otherwise = let s = 1/x'
+                  in  evaluateEvenPolynomialL s
+                        [   r + log x' - 0.5 * s
+                        , - 1/12
+                        ,   1/120
+                        , - 1/252
+                        ,   1/240
+                        , - 1/132
+                        ,  391/32760
+                        ]
+  where
+    γ  = m_eulerMascheroni
+    c  = 12
+    -- Reduce to digamma (x + n) where (x + n) >= c
+    (r, x') = reduce 0 x
+      where
+        reduce !s y
+          | y < c     = reduce (s - 1 / y) (y + 1)
+          | otherwise = (s, y)
+
+
+
+----------------------------------------------------------------
+-- Constants
+----------------------------------------------------------------
+
+-- Coefficients for 18-point Gauss-Legendre integration. They are
+-- used in implementation of incomplete gamma and beta functions.
+coefW,coefY :: U.Vector Double
+coefW = U.fromList [ 0.0055657196642445571, 0.012915947284065419, 0.020181515297735382
+                   , 0.027298621498568734,  0.034213810770299537, 0.040875750923643261
+                   , 0.047235083490265582,  0.053244713977759692, 0.058860144245324798
+                   , 0.064039797355015485,  0.068745323835736408, 0.072941885005653087
+                   , 0.076598410645870640,  0.079687828912071670, 0.082187266704339706
+                   , 0.084078218979661945,  0.085346685739338721, 0.085983275670394821
+                   ]
+coefY = U.fromList [ 0.0021695375159141994, 0.011413521097787704, 0.027972308950302116
+                   , 0.051727015600492421,  0.082502225484340941, 0.12007019910960293
+                   , 0.16415283300752470,   0.21442376986779355,  0.27051082840644336
+                   , 0.33199876341447887,   0.39843234186401943,  0.46931971407375483
+                   , 0.54413605556657973,   0.62232745288031077,  0.70331500465597174
+                   , 0.78649910768313447,   0.87126389619061517,  0.95698180152629142
+                   ]
+{-# NOINLINE coefW #-}
+{-# NOINLINE coefY #-}
+
+trigamma1 :: Double
+trigamma1 = 1.6449340668482264365 -- pi**2 / 6
+
+modErr :: String -> a
+modErr msg = error $ "Numeric.SpecFunctions." ++ msg
 
 
 
 -- $references
 --
+-- * Bernardo, J. (1976) Algorithm AS 103: Psi (digamma)
+--   function. /Journal of the Royal Statistical Society. Series C
+--   (Applied Statistics)/ 25(3):315-317.
+--   <http://www.jstor.org/stable/2347257>
+--
+-- * Cran, G.W., Martin, K.J., Thomas, G.E. (1977) Remark AS R19
+--   and Algorithm AS 109: A Remark on Algorithms: AS 63: The
+--   Incomplete Beta Integral AS 64: Inverse of the Incomplete Beta
+--   Function Ratio. /Journal of the Royal Statistical Society. Series
+--   C (Applied Statistics)/ Vol. 26, No. 1 (1977), pp. 111-114
+--   <http://www.jstor.org/pss/2346887>
+--
 -- * Lanczos, C. (1964) A precision approximation of the gamma
 --   function.  /SIAM Journal on Numerical Analysis B/
 --   1:86&#8211;96. <http://www.jstor.org/stable/2949767>
@@ -574,10 +826,6 @@ n `choose` k
 --   /Journal of the Royal Statistical Society, Series C (Applied Statistics)/
 --   38(2):397&#8211;402. <http://www.jstor.org/stable/2348078>
 --
--- * Shea, B. (1988) Algorithm AS 239: Chi-squared and incomplete
---   gamma integral. /Applied Statistics/
---   37(3):466&#8211;473. <http://www.jstor.org/stable/2347328>
---
 -- * Majumder, K.L., Bhattacharjee, G.P. (1973) Algorithm AS 63: The
 --   Incomplete Beta Integral. /Journal of the Royal Statistical
 --   Society. Series C (Applied Statistics)/ Vol. 22, No. 3 (1973),
@@ -589,9 +837,6 @@ n `choose` k
 --   Vol. 22, No. 3 (1973), pp. 411-414
 --   <http://www.jstor.org/pss/2346798>
 --
--- * Cran, G.W., Martin, K.J., Thomas, G.E. (1977) Remark AS R19
---   and Algorithm AS 109: A Remark on Algorithms: AS 63: The
---   Incomplete Beta Integral AS 64: Inverse of the Incomplete Beta
---   Function Ratio. /Journal of the Royal Statistical Society. Series
---   C (Applied Statistics)/ Vol. 26, No. 1 (1977), pp. 111-114
---   <http://www.jstor.org/pss/2346887>
+-- * Shea, B. (1988) Algorithm AS 239: Chi-squared and incomplete
+--   gamma integral. /Applied Statistics/
+--   37(3):466&#8211;473. <http://www.jstor.org/stable/2347328>
